@@ -7,7 +7,7 @@ use crate::common::{
 use crate::error::{Error, Result};
 use crate::format::{Formatter, LazyFormat, NaiveDateTime};
 use crate::{DateTime, IntervalDT, IntervalYM, Time, Timestamp};
-use std::cmp::{min, Ordering};
+use std::cmp::Ordering;
 use std::convert::TryFrom;
 use std::fmt::Display;
 
@@ -99,11 +99,23 @@ impl Date {
     /// Creates a `Date` from the given year, month, and day.
     #[inline]
     pub const fn try_from_ymd(year: i32, month: u32, day: u32) -> Result<Date> {
-        if Date::is_valid(year, month, day) {
-            Ok(unsafe { Date::from_ymd_unchecked(year, month, day) })
-        } else {
-            Err(Error::OutOfRange)
+        if year < DATE_MIN_YEAR || year > DATE_MAX_YEAR {
+            return Err(Error::DateOutOfRange);
         }
+
+        if month < 1 || month > MONTHS_PER_YEAR {
+            return Err(Error::InvalidMonth);
+        }
+
+        if day < 1 || day > 31 {
+            return Err(Error::InvalidDay);
+        }
+
+        if day > days_of_month(year, month) {
+            return Err(Error::InvalidDate);
+        }
+
+        Ok(unsafe { Date::from_ymd_unchecked(year, month, day) })
     }
 
     /// Checks if the given year, month, and day fields are valid.
@@ -150,7 +162,7 @@ impl Date {
         if is_valid_date(days) {
             Ok(unsafe { Date::from_days_unchecked(days) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::DateOutOfRange)
         }
     }
 
@@ -201,7 +213,7 @@ impl Date {
         let result = self.days().checked_add(days);
         match result {
             Some(d) => Date::try_from_days(d),
-            None => Err(Error::OutOfRange),
+            None => Err(Error::DateOutOfRange),
         }
     }
 
@@ -220,13 +232,7 @@ impl Date {
             new_month = new_month % MONTHS_PER_YEAR as i32 + MONTHS_PER_YEAR as i32;
         }
 
-        let new_day = if day > 28 {
-            min(days_of_month(new_year, new_month as u32), day)
-        } else {
-            day
-        };
-
-        Date::try_from_ymd(new_year, new_month as u32, new_day)
+        Date::try_from_ymd(new_year, new_month as u32, day)
     }
 
     /// `Date` adds `IntervalYM`
@@ -259,7 +265,7 @@ impl Date {
         let result = self.days().checked_sub(days);
         match result {
             Some(d) => Date::try_from_days(d),
-            None => Err(Error::OutOfRange),
+            None => Err(Error::DateOutOfRange),
         }
     }
 
@@ -617,7 +623,7 @@ mod tests {
         let date = generate_date(2001, 12, 31);
         let interval = IntervalDT::try_from_dhms(1, 0, 0, 0, 1).unwrap();
         let expect = generate_ts(2002, 1, 1, 0, 0, 0, 1);
-        assert_eq!(date.clone().add_interval_dt(interval).unwrap(), expect);
+        assert_eq!(date.add_interval_dt(interval).unwrap(), expect);
 
         // Sub negative interval with carry test
         let interval = -IntervalDT::try_from_dhms(1, 0, 0, 0, 1).unwrap();
@@ -637,7 +643,7 @@ mod tests {
         let date = generate_date(1970, 1, 1);
         let interval = -IntervalDT::try_from_dhms(0, 0, 0, 0, 1).unwrap();
         let expect = generate_ts(1969, 12, 31, 23, 59, 59, 999999);
-        assert_eq!(date.clone().add_interval_dt(interval).unwrap(), expect);
+        assert_eq!(date.add_interval_dt(interval).unwrap(), expect);
 
         // Sub positive interval with carry test
         let interval = IntervalDT::try_from_dhms(0, 0, 0, 0, 1).unwrap();
@@ -647,10 +653,10 @@ mod tests {
         let date = generate_date(9999, 12, 31);
         let interval = IntervalDT::try_from_dhms(5, 4, 3, 2, 1).unwrap();
         let expect = generate_ts(9999, 12, 25, 19, 56, 57, 999999);
-        assert_eq!(date.clone().sub_interval_dt(interval).unwrap(), expect);
+        assert_eq!(date.sub_interval_dt(interval).unwrap(), expect);
 
         let interval = IntervalDT::try_from_dhms(1, 0, 0, 0, 1).unwrap();
-        assert!(date.clone().add_interval_dt(interval).is_err());
+        assert!(date.add_interval_dt(interval).is_err());
 
         let interval = IntervalDT::try_from_dhms(12345, 12, 3, 5, 6).unwrap();
         assert!(date.add_interval_dt(interval).is_err());
@@ -658,10 +664,10 @@ mod tests {
         let date = generate_date(1, 1, 1);
         let interval = IntervalDT::try_from_dhms(5, 4, 3, 2, 1).unwrap();
         let expect = generate_ts(1, 1, 6, 4, 3, 2, 1);
-        assert_eq!(date.clone().add_interval_dt(interval).unwrap(), expect);
+        assert_eq!(date.add_interval_dt(interval).unwrap(), expect);
 
         let interval = IntervalDT::try_from_dhms(0, 0, 0, 0, 1).unwrap();
-        assert!(date.clone().sub_interval_dt(interval).is_err());
+        assert!(date.sub_interval_dt(interval).is_err());
 
         let interval = IntervalDT::try_from_dhms(12345, 12, 3, 5, 6).unwrap();
         assert!(date.sub_interval_dt(interval).is_err());
@@ -683,24 +689,6 @@ mod tests {
             generate_ts(2002, 5, 31, 0, 0, 0, 0)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            date.add_interval_ym(interval).unwrap(),
-            generate_ts(2002, 4, 30, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            date.add_interval_ym(interval).unwrap(),
-            generate_ts(2002, 2, 28, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
-        assert_eq!(
-            date.add_interval_ym(interval).unwrap(),
-            generate_ts(2004, 2, 29, 0, 0, 0, 0)
-        );
-
         // Sub negative
         let date = generate_date(2001, 3, 31);
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
@@ -715,24 +703,6 @@ mod tests {
             generate_ts(2002, 5, 31, 0, 0, 0, 0)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2002, 4, 30, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2002, 2, 28, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2004, 2, 29, 0, 0, 0, 0)
-        );
-
         // Sub positive
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
         assert_eq!(
@@ -746,24 +716,6 @@ mod tests {
             generate_ts(2000, 1, 31, 0, 0, 0, 0)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(interval).unwrap(),
-            generate_ts(2000, 2, 29, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(interval).unwrap(),
-            generate_ts(2000, 4, 30, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
-        assert_eq!(
-            date.sub_interval_ym(interval).unwrap(),
-            generate_ts(1999, 2, 28, 0, 0, 0, 0)
-        );
-
         // Add negative
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
         assert_eq!(
@@ -777,24 +729,6 @@ mod tests {
             generate_ts(2000, 1, 31, 0, 0, 0, 0)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            date.add_interval_ym(-interval).unwrap(),
-            generate_ts(2000, 2, 29, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            date.add_interval_ym(-interval).unwrap(),
-            generate_ts(2000, 4, 30, 0, 0, 0, 0)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
-        assert_eq!(
-            date.add_interval_ym(-interval).unwrap(),
-            generate_ts(1999, 2, 28, 0, 0, 0, 0)
-        );
-
         let date = generate_date(2001, 2, 28);
         let interval = IntervalYM::try_from_ym(2, 0).unwrap();
         assert_eq!(
@@ -813,19 +747,6 @@ mod tests {
             generate_ts(1999, 1, 28, 0, 0, 0, 0)
         );
 
-        let date = generate_date(2000, 2, 29);
-        let interval = IntervalYM::try_from_ym(2, 0).unwrap();
-        assert_eq!(
-            date.add_interval_ym(interval).unwrap(),
-            generate_ts(2002, 2, 28, 0, 0, 0, 0)
-        );
-
-        let interval = -IntervalYM::try_from_ym(0, 1).unwrap();
-        assert_eq!(
-            date.add_interval_ym(interval).unwrap(),
-            generate_ts(2000, 1, 29, 0, 0, 0, 0)
-        );
-
         // Boundary test
         let upper_date = generate_date(9999, 12, 31);
         let lower_date = generate_date(1, 1, 1);
@@ -833,6 +754,51 @@ mod tests {
 
         assert!(upper_date.add_interval_ym(interval).is_err());
         assert!(lower_date.sub_interval_ym(interval).is_err());
+
+        // Month day overflow
+        let date = generate_date(2001, 3, 31);
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(date.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(date.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
+        assert!(date.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(date.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(date.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
+        assert!(date.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(date.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(date.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(date.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(date.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(date.add_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(date.add_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(date.add_interval_ym(-interval).is_err());
+
+        let date = generate_date(2000, 2, 29);
+        let interval = IntervalYM::try_from_ym(2, 0).unwrap();
+        assert!(date.add_interval_ym(interval).is_err());
     }
 
     #[test]
