@@ -12,7 +12,7 @@ use crate::{DateTime, Formatter};
 use std::cmp::Ordering;
 use std::convert::TryFrom;
 use std::fmt::Display;
-use std::ops::{Div, Mul, Neg};
+use std::ops::Neg;
 
 const INTERVAL_MAX_YEAR: i32 = 178_000_000;
 const INTERVAL_MAX_DAY: i32 = 100_000_000;
@@ -69,7 +69,7 @@ impl IntervalYM {
         if IntervalYM::is_valid_ym(year, month) {
             Ok(unsafe { IntervalYM::from_ym_unchecked(year, month) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::IntervalOutOfRange)
         }
     }
 
@@ -79,7 +79,7 @@ impl IntervalYM {
         if IntervalYM::is_valid_months(months) {
             Ok(unsafe { IntervalYM::from_months_unchecked(months) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::IntervalOutOfRange)
         }
     }
 
@@ -146,7 +146,7 @@ impl IntervalYM {
         let result = self.months().checked_add(interval.months());
         match result {
             Some(i) => IntervalYM::try_from_months(i),
-            None => Err(Error::OutOfRange),
+            None => Err(Error::IntervalOutOfRange),
         }
     }
 
@@ -160,12 +160,15 @@ impl IntervalYM {
     #[inline]
     pub fn mul_f64(self, number: f64) -> Result<IntervalYM> {
         let months = self.months() as f64;
-        let result = months.mul(number);
+        let result = months * number;
 
-        if !result.is_finite() {
-            return Err(Error::OutOfRange);
+        if result.is_infinite() {
+            Err(Error::NumericOverflow)
+        } else if result.is_nan() {
+            Err(Error::InvalidNumber)
+        } else {
+            IntervalYM::try_from_months(result as i32)
         }
-        IntervalYM::try_from_months(result as i32)
     }
 
     /// `IntervalYM` divides `f64`
@@ -175,12 +178,15 @@ impl IntervalYM {
             return Err(Error::DivideByZero);
         }
         let months = self.months() as f64;
-        let result = months.div(number);
+        let result = months / number;
 
-        if !result.is_finite() {
-            return Err(Error::OutOfRange);
+        if result.is_infinite() {
+            Err(Error::NumericOverflow)
+        } else if result.is_nan() {
+            Err(Error::InvalidNumber)
+        } else {
+            IntervalYM::try_from_months(result as i32)
         }
-        IntervalYM::try_from_months(result as i32)
     }
 }
 
@@ -304,7 +310,7 @@ impl IntervalDT {
         if IntervalDT::is_valid(day, hour, minute, sec, usec) {
             Ok(unsafe { IntervalDT::from_dhms_unchecked(day, hour, minute, sec, usec) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::IntervalOutOfRange)
         }
     }
 
@@ -324,7 +330,7 @@ impl IntervalDT {
         if IntervalDT::is_valid_usecs(usecs) {
             Ok(unsafe { IntervalDT::from_usecs_unchecked(usecs) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::IntervalOutOfRange)
         }
     }
 
@@ -422,7 +428,7 @@ impl IntervalDT {
         let result = self.usecs().checked_add(interval.usecs());
         match result {
             Some(i) => IntervalDT::try_from_usecs(i),
-            None => Err(Error::OutOfRange),
+            None => Err(Error::IntervalOutOfRange),
         }
     }
 
@@ -436,11 +442,15 @@ impl IntervalDT {
     #[inline]
     pub fn mul_f64(self, number: f64) -> Result<IntervalDT> {
         let usecs = self.usecs() as f64;
-        let result = usecs.mul(number).round();
-        if !result.is_finite() {
-            return Err(Error::OutOfRange);
+        let result = usecs * number;
+
+        if result.is_infinite() {
+            Err(Error::NumericOverflow)
+        } else if result.is_nan() {
+            Err(Error::InvalidNumber)
+        } else {
+            IntervalDT::try_from_usecs(result as i64)
         }
-        IntervalDT::try_from_usecs(result as i64)
     }
 
     /// `IntervalDT` divides `f64`
@@ -450,12 +460,15 @@ impl IntervalDT {
             return Err(Error::DivideByZero);
         }
         let usecs = self.usecs() as f64;
-        let result = usecs.div(number).round();
+        let result = usecs / number;
 
-        if !result.is_finite() {
-            return Err(Error::OutOfRange);
+        if result.is_infinite() {
+            Err(Error::NumericOverflow)
+        } else if result.is_nan() {
+            Err(Error::InvalidNumber)
+        } else {
+            IntervalDT::try_from_usecs(result as i64)
         }
-        IntervalDT::try_from_usecs(result as i64)
     }
 
     /// `IntervalDT` subtracts `Time`
@@ -588,12 +601,24 @@ mod tests {
         let interval2 = IntervalYM::parse("178000000-00", "yyyy-mm").unwrap();
         assert_eq!(interval2, interval);
 
+        let interval = -IntervalYM::try_from_ym(1, 0).unwrap();
+        let fmt = format!("{}", interval.format("yy-mm").unwrap());
+        assert_eq!(fmt, "-01-00");
+
+        let interval = IntervalYM::try_from_ym(123, 2).unwrap();
+        let fmt = format!("{}", interval.format("yy-mm").unwrap());
+        assert_eq!(fmt, "123-02");
+
         let interval = -IntervalYM::try_from_ym(178000000, 0).unwrap();
         assert_eq!(interval.extract(), (Negative, 178000000, 0));
         let interval = IntervalYM::try_from_ym(178000000, 0).unwrap().negate();
         assert_eq!(interval.extract(), (Negative, 178000000, 0));
         let fmt = format!("{}", interval.format("yyyy-mm").unwrap());
         assert_eq!(fmt, "-178000000-00");
+
+        let fmt = format!("{}", interval.format("yy-mm").unwrap());
+        assert_eq!(fmt, "-178000000-00");
+
         let interval2 = IntervalYM::parse("-178000000-00", "yyyy-mm").unwrap();
         assert_eq!(interval2, interval);
 
@@ -624,6 +649,14 @@ mod tests {
         let interval2 = IntervalYM::parse("12345-1", "yyyy-mm").unwrap();
         assert_eq!(interval, interval2);
 
+        let interval = IntervalYM::try_from_ym(12345, 1).unwrap();
+        let interval2 = IntervalYM::parse("12345-1", "yy-mm").unwrap();
+        assert_eq!(interval, interval2);
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        let interval2 = IntervalYM::parse("1-1", "yy-mm").unwrap();
+        assert_eq!(interval, interval2);
+
         // Invalid
         assert!(IntervalYM::parse("178000000-1", "yyyy-mm").is_err());
         assert!(IntervalYM::parse("178000001-0", "yyyy-mm").is_err());
@@ -944,7 +977,7 @@ mod tests {
                 .unwrap()
                 .div_f64(-5.1)
                 .unwrap(),
-            -IntervalDT::try_from_dhms(0, 5, 6, 29, 19609).unwrap()
+            -IntervalDT::try_from_dhms(0, 5, 6, 29, 19608).unwrap()
         );
 
         assert_eq!(
@@ -952,7 +985,7 @@ mod tests {
                 .unwrap()
                 .mul_f64(-5.57)
                 .unwrap(),
-            -IntervalDT::try_from_dhms(6, 1, 6, 16, 880028).unwrap()
+            -IntervalDT::try_from_dhms(6, 1, 6, 16, 880027).unwrap()
         );
 
         // Out of range
