@@ -43,7 +43,7 @@ impl Time {
         if Time::is_valid(hour, minute, sec, usec) {
             Ok(unsafe { Time::from_hms_unchecked(hour, minute, sec, usec) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::TimeOutOfRange)
         }
     }
 
@@ -92,7 +92,7 @@ impl Time {
         if is_valid_time(usecs) {
             Ok(unsafe { Time::from_usecs_unchecked(usecs) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::TimeOutOfRange)
         }
     }
 
@@ -354,6 +354,24 @@ mod tests {
             assert_eq!(format!("{}", time.format("FF").unwrap()), "000010");
         }
 
+        // tolerate absence of time
+        {
+            // Absence of time
+            {
+                let time = Time::parse("", "HH24:MI:SS").unwrap();
+                assert_eq!(time, Time::try_from_hms(0, 0, 0, 0).unwrap());
+
+                let time = Time::parse("11", "HH24:MI:SS").unwrap();
+                assert_eq!(time, Time::try_from_hms(11, 0, 0, 0).unwrap());
+
+                let time = Time::parse("11:23", "HH24:MI:SS").unwrap();
+                assert_eq!(time, Time::try_from_hms(11, 23, 0, 0).unwrap());
+
+                let time = Time::parse("11:23:25", "HH24:MI:SS.ff").unwrap();
+                assert_eq!(time, Time::try_from_hms(11, 23, 25, 0).unwrap());
+            }
+        }
+
         // Invalid
         {
             assert!(Time::parse("60", "SS").is_err());
@@ -455,7 +473,7 @@ mod tests {
                 .unwrap()
                 .mul_f64(-5.2)
                 .unwrap(),
-            -IntervalDT::try_from_dhms(0, 5, 22, 39, 600021).unwrap()
+            -IntervalDT::try_from_dhms(0, 5, 22, 39, 600020).unwrap()
         );
 
         assert_eq!(
@@ -463,7 +481,7 @@ mod tests {
                 .unwrap()
                 .div_f64(-5.2)
                 .unwrap(),
-            -IntervalDT::try_from_dhms(0, 0, 23, 40, 1).unwrap()
+            -IntervalDT::try_from_dhms(0, 0, 23, 40, 0).unwrap()
         );
 
         assert_eq!(
@@ -487,7 +505,7 @@ mod tests {
                 .unwrap()
                 .mul_f64(100.1)
                 .unwrap(),
-            IntervalDT::try_from_dhms(8, 13, 18, 58, 400501).unwrap()
+            IntervalDT::try_from_dhms(8, 13, 18, 58, 400500).unwrap()
         );
 
         // Round
@@ -504,7 +522,7 @@ mod tests {
                 .unwrap()
                 .mul_f64(-5.57)
                 .unwrap(),
-            -IntervalDT::try_from_dhms(0, 11, 25, 28, 880028).unwrap()
+            -IntervalDT::try_from_dhms(0, 11, 25, 28, 880027).unwrap()
         );
 
         // Out of range
