@@ -107,7 +107,7 @@ impl Timestamp {
         if is_valid_timestamp(usecs) {
             Ok(unsafe { Timestamp::from_usecs_unchecked(usecs) })
         } else {
-            Err(Error::OutOfRange)
+            Err(Error::DateOutOfRange)
         }
     }
 
@@ -117,7 +117,7 @@ impl Timestamp {
         let result = self.usecs().checked_add(interval.usecs());
         match result {
             Some(ts) => Timestamp::try_from_usecs(ts),
-            None => Err(Error::OutOfRange),
+            None => Err(Error::DateOutOfRange),
         }
     }
 
@@ -142,13 +142,16 @@ impl Timestamp {
     #[inline]
     pub fn add_days(self, days: f64) -> Result<Timestamp> {
         let microseconds = (days * USECONDS_PER_DAY as f64).round();
-        if !microseconds.is_finite() {
-            return Err(Error::OutOfRange);
-        }
-        let result = self.usecs().checked_add(microseconds as i64);
-        match result {
-            Some(d) => Timestamp::try_from_usecs(d),
-            None => Err(Error::OutOfRange),
+        if microseconds.is_infinite() {
+            Err(Error::NumericOverflow)
+        } else if microseconds.is_nan() {
+            Err(Error::InvalidNumber)
+        } else {
+            let result = self.usecs().checked_add(microseconds as i64);
+            match result {
+                Some(d) => Timestamp::try_from_usecs(d),
+                None => Err(Error::DateOutOfRange),
+            }
         }
     }
 
@@ -469,6 +472,23 @@ mod tests {
                 assert_eq!(ts, ts2)
             }
 
+            // Absence of time
+            {
+                let timestamp = Timestamp::parse("2020-11-12", "YYYY-MM-DD HH24:MI:SS").unwrap();
+                assert_eq!(timestamp, generate_ts(2020, 11, 12, 0, 0, 0, 0));
+
+                let timestamp = Timestamp::parse("2020-11-12 11", "YYYY-MM-DD HH24:MI:SS").unwrap();
+                assert_eq!(timestamp, generate_ts(2020, 11, 12, 11, 0, 0, 0));
+
+                let timestamp =
+                    Timestamp::parse("2020-11-12 11:23", "YYYY-MM-DD HH24:MI:SS").unwrap();
+                assert_eq!(timestamp, generate_ts(2020, 11, 12, 11, 23, 0, 0));
+
+                let timestamp =
+                    Timestamp::parse("2020-11-12 11:23:25", "YYYY-MM-DD HH24:MI:SS.ff").unwrap();
+                assert_eq!(timestamp, generate_ts(2020, 11, 12, 11, 23, 25, 0));
+            }
+
             // Short format
             {
                 let ts = generate_ts(1234, 8, 6, 7, 8, 9, 10);
@@ -543,6 +563,38 @@ mod tests {
                 );
             }
 
+            // fraction rounding and etc
+            {
+                assert_eq!(
+                    Timestamp::parse(".12345", ".ff").unwrap(),
+                    generate_ts(1, 1, 1, 0, 0, 0, 123450)
+                );
+                assert_eq!(
+                    Timestamp::parse(".123456789", ".ff").unwrap(),
+                    generate_ts(1, 1, 1, 0, 0, 0, 123457)
+                );
+                assert_eq!(
+                    Timestamp::parse(".12345678", ".ff").unwrap(),
+                    generate_ts(1, 1, 1, 0, 0, 0, 123457)
+                );
+                assert_eq!(
+                    Timestamp::parse(".1234567", ".ff7").unwrap(),
+                    generate_ts(1, 1, 1, 0, 0, 0, 123457)
+                );
+                assert!(Timestamp::parse(".12345678", ".ff7").is_err());
+                assert_eq!(
+                    Timestamp::parse(".123456", ".ff6").unwrap(),
+                    generate_ts(1, 1, 1, 0, 0, 0, 123456)
+                );
+                assert!(Timestamp::parse(".123456789", ".ff2").is_err());
+
+                let timestamp = generate_ts(1, 2, 3, 4, 5, 6, 123456);
+                assert_eq!(format!("{}", timestamp.format("ff6").unwrap()), "123456");
+                assert_eq!(format!("{}", timestamp.format("ff").unwrap()), "123456");
+                assert_eq!(format!("{}", timestamp.format("ff9").unwrap()), "123456000");
+                assert_eq!(format!("{}", timestamp.format("ff5").unwrap()), "12345");
+            }
+
             // Day parse check
             {
                 let ts = generate_ts(2021, 4, 22, 3, 4, 5, 6);
@@ -795,24 +847,6 @@ mod tests {
             generate_ts(2002, 5, 31, 12, 5, 6, 7)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(interval).unwrap(),
-            generate_ts(2002, 4, 30, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(interval).unwrap(),
-            generate_ts(2002, 2, 28, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(interval).unwrap(),
-            generate_ts(2004, 2, 29, 12, 5, 6, 7)
-        );
-
         // Sub negative
         let ts = generate_ts(2001, 3, 31, 12, 5, 6, 7);
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
@@ -827,24 +861,6 @@ mod tests {
             generate_ts(2002, 5, 31, 12, 5, 6, 7)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2002, 4, 30, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2002, 2, 28, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(-interval).unwrap(),
-            generate_ts(2004, 2, 29, 12, 5, 6, 7)
-        );
-
         // Sub positive
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
         assert_eq!(
@@ -858,24 +874,6 @@ mod tests {
             generate_ts(2000, 1, 31, 12, 5, 6, 7)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(interval).unwrap(),
-            generate_ts(2000, 2, 29, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(interval).unwrap(),
-            generate_ts(2000, 4, 30, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
-        assert_eq!(
-            ts.sub_interval_ym(interval).unwrap(),
-            generate_ts(1999, 2, 28, 12, 5, 6, 7)
-        );
-
         // Add negative
         let interval = IntervalYM::try_from_ym(0, 2).unwrap();
         assert_eq!(
@@ -889,24 +887,6 @@ mod tests {
             generate_ts(2000, 1, 31, 12, 5, 6, 7)
         );
 
-        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(-interval).unwrap(),
-            generate_ts(2000, 2, 29, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(-interval).unwrap(),
-            generate_ts(2000, 4, 30, 12, 5, 6, 7)
-        );
-
-        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
-        assert_eq!(
-            ts.add_interval_ym(-interval).unwrap(),
-            generate_ts(1999, 2, 28, 12, 5, 6, 7)
-        );
-
         // Boundary test
         let upper_ts = generate_ts(9999, 12, 31, 23, 59, 59, 999999);
         let lower_ts = generate_ts(1, 1, 1, 0, 0, 0, 0);
@@ -914,6 +894,46 @@ mod tests {
 
         assert!(upper_ts.add_interval_ym(interval).is_err());
         assert!(lower_ts.sub_interval_ym(interval).is_err());
+
+        // Month day overflow
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(ts.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(ts.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
+        assert!(ts.add_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(ts.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(ts.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 11).unwrap();
+        assert!(ts.sub_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(ts.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(ts.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(ts.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(ts.sub_interval_ym(interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(1, 1).unwrap();
+        assert!(ts.add_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(0, 11).unwrap();
+        assert!(ts.add_interval_ym(-interval).is_err());
+
+        let interval = IntervalYM::try_from_ym(2, 1).unwrap();
+        assert!(ts.add_interval_ym(-interval).is_err());
     }
 
     #[test]
